<?php
// $Id: uts.analysis.inc,v 1.9 2008/07/06 05:40:39 boombatower Exp $
/**
 * @file
 * Provide data analysis pages for the Usability Testing Suite.
 *
 * Copyright 2008 by Jimmy Berry ("boombatower", http://drupal.org/user/214218)
 */

/**
 * Menu callback: Display list of studies to analyze, list of sessions to
 * analyze, or display particular plugin generated analysis.
 *
 * @param interger $study_nid Current study NID.
 * @return HTML Output.
 */
function uts_analysis($study_nid = NULL) {
  if ($study_nid) {
    $sessions = uts_session_load_all($study_nid);
    $analysis_plugins = uts_data_analysis_list();

    // Look for session ID or plug-in to display.
    $session_id = NULL;
    $plugin = NULL;
    if (in_array(arg(4), $sessions)) {
      $session_id = arg(4);
      if (array_key_exists(arg(5), $analysis_plugins)) {
        $plugin = arg(5);
      }
    }
    else if (array_key_exists(arg(4), $analysis_plugins)) {
      $plugin = arg(4);
    }

    uts_analysis_local_tasks_add($study_nid, $session_id, $plugin, $analysis_plugins);

    if ($plugin) {
      // Get plug-in mode.
      $mode = ($session_id && $plugin ? arg(6) : arg(5));
      $plugin_modes = $analysis_plugins[$plugin];
      $mode = ($mode ? $mode : $plugin_modes[0]);

      // Display data analysis from plug-in.
      if ($session_id) {
        list($start, $stop) = uts_data_get_timestamps($session_id); // TODO Provide timestamp filter.
      }
      else {
        list($start, $stop) = array(NULL, NULL);
      }
      $elements = uts_data_analysis_display($plugin, $mode, $study_nid, $session_id, $start, $stop);
      uasort($elements, "element_sort");

      // Output all the elements which have been sorted by weight.
      $output = '';
      foreach ($elements as $element) {
        $output .= $element['#data'];
      }

      return $output;
    }

    $headers = array(t('Name'), t('Status'), t('Total time'), t('Operations'));
    $rows = array();
    foreach ($sessions as $session_id) {
      $row = array();
      $session = uts_session_load($session_id);

      $row[] = $session->session_id;
      $row[] = ($session->complete ? t('complete') : t('incomplete'));
      $row[] = format_interval(uts_data_get_total_time($session->session_id));
      $row[] = uts_render_operations(uts_session_operations($session));
      $rows[] = $row;
    }
    return t('<p>Use the analysis tabs to view overal study analysis, or select an individual session for analysis.</p>
              !sessions', array('!sessions' => theme('table', $headers, $rows)));
  }
  else {
    $studies = uts_studies_load(UTS_STUDY_STATUS_ACTIVE);
    $list = array();
    foreach ($studies as $study) {
      $list[] = t('!title<br />@description',
                   array('!title' => l($study->title, 'admin/uts/analyze/' . $study->nid), '@description' => t($study->body)));
    }
    return t('<p>Select a study to analyze.</p>!studies', array('!studies' => theme('item_list', $list)));
  }
}

/**
 * Add local tasks for analysis.
 *
 * @param interger $study_nid Current study NID.
 * @param string $session_id Current sessiond ID.
 * @param string $plugin Current data analysis plug-in.
 * @param array $analysis_plugins List a analysis plug-ins and modes.
 */
function uts_analysis_local_tasks_add($study_nid, $session_id, $plugin, $analysis_plugins) {
  if ($study_nid) {
    uts_local_tasks(0, t('Sessions'), "admin/uts/analyze/$study_nid/sessions", !$plugin);

    // Display plug-in links.
    $base_url = "admin/uts/analyze/$study_nid" . ($session_id ? "/$session_id" : '');
    $collection_plugins = uts_data_collection_list();
    foreach ($analysis_plugins as $module => $modes) {
      uts_local_tasks(0, $collection_plugins[$module]['name'], "$base_url/$module", $module == $plugin);
    }

    // If plug-in selected display mode links.
    if ($plugin) {
      $base_url .= "/$plugin";
      $plugin_modes = $analysis_plugins[$plugin];
      if (count($plugin_modes) > 1) {
        // Display modes.
        foreach ($plugin_modes as $plugin_mode) {
          uts_local_tasks(1, t($plugin_mode), "$base_url/$plugin_mode", !$plugin);
        }
      }
    }
  }
}

/**
 * Get an array containing the session operation links.
 *
 * @param $session Session object.
 * @return Array of links.
 */
function uts_session_operations($session) {
  $links = array();
  if ($session->complete) {
    $links['analyze'] = array(
      'title' => t('analyze'),
      'href' => "admin/uts/analyze/$session->study_nid/$session->session_id"
    );
  }
  $links['session_destroy'] = array(
    'title' => t('destroy'),
    'href' => "admin/uts/analyze/$session->study_nid/$session->session_id/destroy"
  );
  return $links;
}

/**
 * Menu callback -- ask for confirmation of session destruction then redirect
 * to appropriate overview page.
 */
function uts_analysis_destroy_confirm(&$form_state, $session_id) {
  $redirect = 'admin/uts/analyze/' . arg(3);

  // Inject form values.
  $form['session_id'] = array(
    '#type' => 'value',
    '#value' => $session_id,
  );
  $form['redirect'] = array(
    '#type' => 'value',
    '#value' => $redirect,
  );

  return confirm_form($form,
    t('Are you sure you want to delete %session?', array('%session' => $session_id)),
    isset($_GET['destination']) ? $_GET['destination'] : $redirect,
    t('This action cannot be undone.'),
    t('Delete'),
    t('Cancel')
  );
}

/**
 * Execute session destruction.
 */
function uts_analysis_destroy_confirm_submit($form, &$form_state) {
  if ($form_state['values']['confirm']) {
    uts_session_destory($form_state['values']['session_id']);
  }

  // Set the redirect to the pre-determined location.
  $form_state['redirect'] = $form_state['values']['redirect'];
}
